unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids, uCrystalApi, ExtCtrls;

const
    CrystalNameType : array [PE_PF_NUMBER..PE_PF_TIME] of String = ('Number','Currency','Boolean','Date','String','DateTime','Time');

type
  TFrmMain = class(TForm)
    lblPathReport: TLabel;
    edtPathReport: TEdit;
    btnPathReport: TButton;
    btnReportParams: TButton;
    btnReportPreview: TButton;
    grReport: TStringGrid;
    btnQuit: TButton;
    dlgOpen: TOpenDialog;
    pnlPreview: TPanel;
    lblPreview: TLabel;
    lblParams: TLabel;
    chkWindow: TCheckBox;
    procedure btnPathReportClick(Sender: TObject);
    procedure btnQuitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnReportPreviewClick(Sender: TObject);
    procedure btnReportParamsClick(Sender: TObject);
  private
    FHandleJob: LongInt;
    procedure DoClearGrid;
    function  FormatCrystalValue(Value: PEValueInfo): String;
  public
    { Public declarations }
  end;


var
  FrmMain: TFrmMain;

implementation

{$R *.dfm}

procedure TFrmMain.btnPathReportClick(Sender: TObject);
begin
    if dlgOpen.Execute then
        begin
            edtPathReport.Text := dlgOpen.FileName;
            btnReportParams.Enabled := True;
            btnReportPreview.Enabled := True;
        end;
end;

procedure TFrmMain.btnQuitClick(Sender: TObject);
begin
    Application.Terminate;
end;

procedure TFrmMain.FormCreate(Sender: TObject);
begin
    with grReport do
        begin
            Cells[0, 0]:= '';
            Cells[1, 0]:= '';
            Cells[2, 0]:= '  ';
        end;

    PEOpenEngine;

end;

procedure TFrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    PECloseEngine;
end;

procedure TFrmMain.btnReportPreviewClick(Sender: TObject);
var
    FWindow: THandle;
    lt: PELogOnInfo;
begin

    if chkWindow.Checked then
        FWindow:= 0
    else
        FWindow:= pnlPreview.Handle;

    FHandleJob:= PEOpenPrintJob(PChar(edtPathReport.Text));

    FillChar(lt, SizeOf(PELogOnInfo), 0);
    lt.StructSize := SizeOf(PELogOnInfo);
    PEGetNthTableLogOnInfo(FHandleJob, 0, lt);

    StrPCopy(@lt.ServerName, ExtractFilePath(edtPathReport.Text) + 'source_db.mdb');
    PESetNthTableLogOnInfo(FHandleJob, 0, lt, False);

        PEOutputToWindow(FHandleJob, PChar(TForm(Self).Caption), 0, 0, 0, 0, 0, FWindow);
        PEStartPrintJob(FHandleJob, True);
    PEClosePrintJob(FHandleJob);

end;

procedure TFrmMain.btnReportParamsClick(Sender: TObject);
var
    ParameterInfo: PEParameterFieldInfo;
    ValueInfo: PEValueInfo;
    i, CountParams: SmallInt;
begin
    DoClearGrid;

    ParameterInfo.StructSize:= SizeOf(PEParameterFieldInfo);
    ValueInfo.StructSize:= SizeOf(PEValueInfo);


    FHandleJob:= PEOpenPrintJob(PChar(edtPathReport.Text));

    CountParams:= PEGetNParameterFields(FHandleJob);

    if CountParams <> -1 then
        begin
            grReport.RowCount := CountParams + 1;
            for i:= 0 to CountParams - 1 do
                begin
                    PEGetNthParameterField(FHandleJob, i, ParameterInfo);
                        ValueInfo.ValueType := ParameterInfo.valueType;
                    PEConvertPFInfoToVInfo(@ParameterInfo.DefaultValue,
                                           ValueInfo.ValueType,
                                           ValueInfo);
                    with ParameterInfo do
                        begin
                            grReport.Cells[0, i + 1]:= Name;
                            grReport.Cells[1, i + 1]:= CrystalNameType[ValueType];
                            grReport.Cells[2, i + 1]:= FormatCrystalValue(ValueInfo);
                        end;
                end;
        end;

    PEClosePrintJob(FHandleJob);

end;

procedure TFrmMain.DoClearGrid;
var
    i: SmallInt;
begin
    with grReport do
        begin
            RowCount := 2;
            for i:= 0 to 2 do
                Cells[i, 1]:= '';
        end;
end;

function TFrmMain.FormatCrystalValue(Value: PEValueInfo): String;
var
    s: String;
begin
    s := '';

    case Value.ValueType of
        PE_VI_NUMBER:
            s:= FloatToStr(Value.viNumber);
        PE_VI_CURRENCY:
            s:= FloatToStr(Value.viCurrency);
        PE_VI_BOOLEAN:
            if Value.viBoolean then
                s:= 'True'
            else
                s:= 'False';
        PE_VI_DATE:
            s:= Format('%d.%d.%d',[Value.viDate[2],Value.viDate[1], Value.viDate[0]]);
        PE_VI_STRING:
            s:= StrPas(Value.viString);
        PE_VI_DATETIME:
            s:= Format('%d.%d.%d %d:%d',[Value.viDateTime[2],Value.viDateTime[1], Value.viDateTime[0], Value.viDateTime[3], Value.viDateTime[4], Value.viDateTime[5]]);
        PE_VI_TIME:
            s:= Format('%d:%d:%d',[Value.viTime[0],Value.viTime[1], Value.viTime[2]]);
        PE_VI_INTEGER:
            s:= IntToStr(Value.viInteger);
        PE_VI_CHAR:
            s:= Value.viC;
        PE_VI_LONG:
            s:= IntToStr(Value.viLong);
    end;

    Result:= s;
end;

end.
